from pathlib import Path

_dataset_dir = Path(__file__).parent / "dataset"
SEGMENT_DIR = _dataset_dir / "media" / "segments"
MEDIA_DIR = _dataset_dir / "media"
ANNOTATIONS_DIR = _dataset_dir / "annotations"
