__title__ = "epic-kitchens"
__description__ = "EPIC Kitchens action dataset support library"
__version__ = "1.6.3"
__author__ = "EPIC KITCHENS"
__author_email__ = "uob-epic-kitchens2018@bristol.ac.uk"
__license__ = "MIT"
__copyright__ = "Copyright 2018 EPIC Kitchens"
