""" Dataset Adapters for GulpIO.

This module contains two adapters for 'gulping' both RGB and flow frames
which can then be used with the :class:`EpicVideoDataset` classes.
"""

from . import adapter
