""" ``epic-kitchens``

This library contains a variety of useful classes and functions for performing common operations on
the `EPIC Kitchens`_ dataset.

.. _`EPIC Kitchens`: https://epic-kitchens.github.io/

"""

from . import gulp, dataset, preprocessing
from .__version__ import __title__, __description__, __version__
from .__version__ import __author__, __author_email__, __license__, __copyright__
